﻿using Dynamicweb.DataIntegration;
using Dynamicweb.Extensibility.AddIns;
using Dynamicweb.Extensibility.Editors;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Web;
using Microsoft.Xrm.Sdk;
using Dynamicweb.DataIntegration.Integration;
using System.ServiceModel.Description;
using Dynamicweb.DataIntegration.Providers.XmlProvider;
using Dynamicweb.DataIntegration.Providers.UserProvider;
using Dynamicweb.Extensibility.Notifications;
using Dynamicweb.Core.Helpers;
using Dynamicweb.DataIntegration.Providers.CRMProvider;
using Dynamicweb.DataIntegration.ProviderHelpers;
using CrmIntegration.Helpers;

namespace CrmIntegration
{
    [AddInName("Crm AddIn"), AddInLabel("Crm AddIn"), AddInDescription("Crm AddIn"), AddInIgnore(false)]
    public class CrmAddIn : BatchIntegrationScheduledTaskAddin, IDropDownOptions
    {
        [AddInParameter("Crm web service url"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public string WebServiceUrl { get; set; }

        [AddInParameter("User name"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public string UserName { get; set; }

        [AddInParameter("Domain"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public string Domain { get; set; }

        [AddInParameter("Password"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;password=true;")]
        public string Password { get; set; }

        [AddInParameter("Contact columns(comma separated)"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;TextArea=True;style=height:120px;")]
        public string ContactsColumns { get; set; }

        [AddInParameter("Users import activity"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;")]
        public string UsersImportActivity { get; set; }

        [AddInParameter("Destination group"), AddInParameterEditor(typeof(UserGroupParameterEditor), "")]
        public string DestinationFolder { get; set; }

        [AddInParameter("Export emails to CRM"), AddInParameterEditor(typeof(YesNoParameterEditor), "")]
        public bool ExportEmailsToCRM { get; set; }

        [AddInParameter("Export emails information activity"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;")]
        public string MarketingExportActivity { get; set; }        

        [AddInParameter("Email Allowed Consent Activity"), AddInParameterEditor(typeof(DropDownParameterEditor), "NewGUI=true;")]
        public string EmailAllowedConsentActivity { get; set; }

        [AddInParameter("Forms for editors: Subject download field id"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public string SubjectDownloadId { get; set; }
        [AddInParameter("Forms for editors: Subject download timestamp"), AddInParameterEditor(typeof(TextParameterEditor), "inputClass=NewUIinput;")]
        public string DownloadTimestampId { get; set; }

        [AddInParameter("Forms for editors: Register new user for CRM"), AddInParameterEditor(typeof(DropDownParameterEditor), "inputClass=NewUIinput;")]
        public string RegisterNewUserForm { get; set; }

        internal ClientCredentials ClientCredentials
        {
            get
            {
                ClientCredentials credentials = new ClientCredentials();
                if (string.IsNullOrEmpty(Domain))
                {
                    credentials.UserName.UserName = UserName;
                    credentials.UserName.Password = Password;
                }
                else
                {
                    credentials.Windows.ClientCredential = new System.Net.NetworkCredential(UserName, Password, Domain);
                }
                return credentials;
            }
        }

        private List<string> ContactColumnsList
        {
            get
            {
                return ContactsColumns.Split(new char[] { ',' }).Select(c => c.Trim()).ToList();
            }
        }

        public override bool Run()
        {
            SetupLogging();
            bool result = true;
            string logFileFullPath = string.Empty;
            string error = string.Empty;

            try
            {                
                if (!string.IsNullOrEmpty(MarketingExportActivity) && ExportEmailsToCRM)
                {
                    logFileFullPath = HttpContext.Current.Server.MapPath("/Files/System/Log/DataIntegration/" + MarketingExportActivity.Replace(";", "--") + DateTime.Now.ToString("yyyyMMdd-HHmmssFFFFFFF") + ".log");
                    result = ExportMarketingInfoToCRM(logFileFullPath, out error);
                }
                if (!string.IsNullOrEmpty(UsersImportActivity))
                {
                    Logger.Log("Start getting contacts from Crm");                    
                    EntityCollection contacts = CrmConnector.GetContacts(new Uri(WebServiceUrl), ClientCredentials, ContactColumnsList);
                    Logger.Log("Finish getting contacts from Crm");
                    if (contacts != null && contacts.Entities != null && contacts.Entities.Count() > 0)
                    {
                        string importUsersError = string.Empty;
                        logFileFullPath = HttpContext.Current.Server.MapPath("/Files/System/Log/DataIntegration/" + UsersImportActivity.Replace(";", "--") + DateTime.Now.ToString("yyyyMMdd-HHmmssFFFFFFF") + ".log");
                        bool importUsersResult = ImportUsers(contacts, logFileFullPath, out importUsersError);
                        result = result ? importUsersResult : result;
                        error = string.IsNullOrEmpty(error) ? importUsersError : string.Format("{0}{1}{2}", error, Environment.NewLine, importUsersError);
                    }
                    else
                    {
                        Logger.Log("No Crm users to import");
                    }
                    contacts = null;
                }
            }
            catch (Exception ex)
            {
                error = ex.Message;
            }
            finally
            {
                if (!result)
                {
                    string activityLog = System.IO.File.ReadAllText(logFileFullPath, Encoding.UTF8);
                    SendMail(activityLog, Dynamicweb.Content.Files.Information.MessageType.Error);
                }
                else if (error != "")
                {
                    //Send mail with error
                    result = false;
                    SendMail(error, Dynamicweb.Content.Files.Information.MessageType.Error);
                }
                else
                {
                    SendMail(Dynamicweb.SystemTools.Translate.Translate("Scheduled task completed successfully"));
                }
                if (System.IO.File.Exists(logFileFullPath))
                {
                    AddFileContentToLog(new List<string>() { logFileFullPath });
                }
                if (!string.IsNullOrEmpty(error))
                    Logger.Log(error);
            }
            WriteTaskResultToLog(result);
            return result;
        }        

        private bool ImportUsers(EntityCollection users, string logFileFullPath, out string error)
        {
            Logger.Log("Start import users");
            bool result = true;
            error = string.Empty;
            if (!string.IsNullOrEmpty(UsersImportActivity))
            {
                Job userJobToRun = new Job(HttpContext.Current.Server.MapPath("/Files/" + Dynamicweb.Content.Files.FilesAndFolders.GetFilesFolderName() + "/Integration/jobs/") + UsersImportActivity + ".xml", System.IO.Path.GetFileName(logFileFullPath));
                if (userJobToRun.Source != null && userJobToRun.Destination != null)
                {
                    if (userJobToRun.Source is XmlProvider && userJobToRun.Destination is UserProvider)
                    {
                        Hashtable accounts = UserInfoHelper.GetAccountsHashTable(CrmConnector.GetAccounts(new Uri(WebServiceUrl), ClientCredentials));

                        string xml = XmlFormatter.GetImportXml(users, DestinationFolder.Replace("GRP_", ""), ContactColumnsList, accounts).OuterXml;
                        if (!string.IsNullOrEmpty(xml))
                        {
                            ((XmlProvider)userJobToRun.Source).WriteToSourceFile(xml);
                            try
                            {
                                result = userJobToRun.Run();
                            }
                            catch (Exception ex)
                            {
                                result = false;
                                error = ex.Message;
                                TextFileHelper.WriteTextFile(string.Format("{0}: Error occured: {1}{2}", DateTime.Now.ToString(), ex.Message, Environment.NewLine), logFileFullPath, true);
                            }
                            NotificationManager.Notify(Dynamicweb.DataIntegration.Integration.Notifications.Integration.JobFinished, new Dynamicweb.DataIntegration.Integration.Notifications.Integration.JobFinishedIntegrationArgs(result, logFileFullPath, userJobToRun));
                        }
                        else
                        {
                            throw new Exception("Xml returned from the Crm web service: is empty");
                        }                        
                    }
                    else
                    {
                        if (!(userJobToRun.Source is XmlProvider))
                        {
                            throw new Exception(string.Format("Source Provider in the '{0}' activity is not XmlProvider", UsersImportActivity));
                        }
                        if (!(userJobToRun.Destination is UserProvider))
                        {
                            throw new Exception(string.Format("Destination Provider in the '{0}' activity is not UserProvider", UsersImportActivity));
                        }
                    }
                }
                else
                {
                    throw new Exception("User job creation failed.");
                }
            }
            else
            {
                Logger.Log("No User Import activity selected. No users imported");
            }
            Logger.Log("Finish import users");
            return result;
        }

        private bool ExportMarketingInfoToCRM(string logFileFullPath, out string error)
        {
            Logger.Log("Start export marketing info to crm");
            bool ret = true;
            error = string.Empty;
            if (!string.IsNullOrEmpty(MarketingExportActivity))
            {
                Job job = new Job(HttpContext.Current.Server.MapPath("/Files/" + Dynamicweb.Content.Files.FilesAndFolders.GetFilesFolderName() + "/Integration/jobs/") + MarketingExportActivity + ".xml", System.IO.Path.GetFileName(logFileFullPath));
                if (job != null && job.Source != null && job.Source is CRMProvider)
                {
                    try
                    {
                        CRMProvider provider = job.Source as CRMProvider;
                        Logger.Log("Start GetExistingActivities");
                        EntityCollection existingActivities = CrmConnector.GetExistingActivities(new Uri(WebServiceUrl), ClientCredentials);
                        Dictionary<string, Entity> existingActivitiesDictionary = EmailMarketingHelper.GetActivitiesInfo(existingActivities);
                        Logger.Log("Finish GetExistingActivities");
                        if (ExportEmailsToCRM)
                        {
                            Logger.Log("Start GetEmailsInfo");
                            List<EmailInfo> emails = provider.GetEmailsInfo();
                            Logger.Log("Start export email to crm");
                            ExportEmails(emails, existingActivitiesDictionary, new Uri(WebServiceUrl), ClientCredentials);
                            Logger.Log("Finish export email to crm");                            
                            provider.UpdateEmailsLastExportDate(emails.Select(e => e.EmailID).ToList());
                            Logger.Log("Finish update exported emails in Dynamicweb");
                        }
                    }
                    catch (Exception ex)
                    {
                        ret = false;
                        error = ex.Message;
                        TextFileHelper.WriteTextFile(string.Format("{0}: Error occured: {1}{2}", DateTime.Now.ToString(), ex.Message, Environment.NewLine), logFileFullPath, true);
                    }
                    NotificationManager.Notify(Dynamicweb.DataIntegration.Integration.Notifications.Integration.JobFinished, new Dynamicweb.DataIntegration.Integration.Notifications.Integration.JobFinishedIntegrationArgs(ret, logFileFullPath, job));
                }
                else
                {
                    if (job != null && job.Source != null)
                    {
                        throw new Exception(string.Format("Source Provider in the '{0}' activity is not CRMProvider", MarketingExportActivity));
                    }
                    else
                    {
                        throw new Exception("Marketing job creation failed.");
                    }
                }
            }
            else
            {
                Logger.Log("No Marketing Import activity selected. No leads or emails exported");
                ret = false;
            }
            Logger.Log("Finish export marketing info to crm");
            return ret;
        }

        private void ExportEmails(List<EmailInfo> emails, Dictionary<string, Entity> existingActivities, Uri uri, ClientCredentials credentials)
        {
            if (emails != null && emails.Count > 0)
            {
                Logger.Log("Start getting contacts from Crm");
                EntityCollection contacts = CrmConnector.GetContacts(uri, credentials, new List<string>() { "ownerid" });
                Dictionary<Guid, Guid?> contactOwnerDictionary = new Dictionary<Guid, Guid?>();                
                foreach(Entity contact in contacts.Entities)
                {
                    if(contact != null && !contactOwnerDictionary.ContainsKey(contact.Id))
                    {                        
                        string owner = contact.GetAttributeValue("ownerid");
                        Guid ownerId = Guid.Empty;
                        contactOwnerDictionary.Add(contact.Id, Guid.TryParse(owner, out ownerId) ? (Guid?)ownerId : null);
                    }
                }
                Logger.Log("Finish getting contacts from Crm");
                foreach (EmailInfo ei in emails)
                {
                    Guid contactID = Guid.Empty;
                    if (!string.IsNullOrEmpty(ei.ExternalUserID))
                    {
                        Guid.TryParse(ei.ExternalUserID, out contactID);
                    }
                    if (contactID != Guid.Empty)
                    {
                        string key = EmailMarketingHelper.GetEmailDWKey(ei.EmailID.ToString(), ei.ExternalUserID);
                        if (existingActivities.ContainsKey(key))
                        {
                            Entity activity = existingActivities[key];
                            Tuple<Guid, int, string, string> value = EmailMarketingHelper.GetActivityInfo(activity);
                            if (value != null)
                            {
                                //1 Completed
                                //2 Canceled
                                //skip updating Closed or Canceled activities as it causes exception in CRM
                                if (value.Item2 != 1 && value.Item2 != 2)
                                {
                                    string subject = EmailMarketingHelper.GetEmailActivitySubject(ei.EmailSendCompleteDate);
                                    string description = EmailMarketingHelper.GetEmailActivityBody(ei);
                                    if (string.Compare(subject, value.Item3, true) != 0 || string.Compare(description, value.Item4, true) != 0)
                                    {
                                        Guid activityID = value.Item1;
                                        CrmConnector.AddOrUpdateAttribute(activity, "subject", subject);
                                        CrmConnector.AddOrUpdateAttribute(activity, "description", description);
                                        CrmConnector.Update(activity, uri, credentials);                                        
                                    }
                                }
                            }
                        }
                        else
                        {
                            Guid? ownerId;
                            if(contactOwnerDictionary.TryGetValue(contactID, out ownerId))
                            {
                                CrmConnector.AddActivity(contactID, EmailMarketingHelper.GetEmailActivitySubject(ei.EmailSendCompleteDate), EmailMarketingHelper.GetEmailActivityBody(ei), key,
                                uri, credentials, ownerId);
                            }
                            else
                            {
                                CrmConnector.AddActivity((Guid?)null, EmailMarketingHelper.GetEmailActivitySubject(ei.EmailSendCompleteDate), EmailMarketingHelper.GetEmailActivityBody(ei), key,
                                uri, credentials, (Guid?)null);
                            }                            
                        }
                    }
                    else
                    {
                        Logger.Log(string.Format("Email Subject='{0}' ID='{1}' has not valid contact GUID='{2}'.", ei.EmailSubject, ei.EmailID, ei.ExternalUserID));
                    }
                }
            }
        }

        public Hashtable GetOptions(string dropdownName)
        {
            var options = new Hashtable();
            switch (dropdownName)
            {
                case "Users import activity":
                case "Export emails information activity":
                    foreach (string jobname in GetActivitiesList())
                    {
                        options.Add(jobname, jobname);
                    }
                    break;
                case "Email Allowed Consent Activity":
                    Dynamicweb.DataProcessing.ActivityService service = new Dynamicweb.DataProcessing.ActivityService();
                    foreach(var activity in service.GetActivities())
                    {
                        options.Add(activity.Id, activity.Name);
                    }
                    break;
                case "Forms for editors: Register new user for CRM":
                    foreach(var form in Dynamicweb.Forms.Form.GetForms())
                    {
                        options.Add(form.ID, form.Name);
                    }
                    break;
                default:
                    break;
            }
            return options;
        }
    }
}
