﻿using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Client;
using System.ServiceModel.Description;
using System;
using Microsoft.Xrm.Sdk.Query;
using System.Collections.Generic;

namespace CrmIntegration
{
    public class CrmConnector
    {
        private static readonly Guid PartnerSupportDynamicwebGuid = new Guid("1EEC55F9-BEA1-E611-80F9-5065F38A4A61");//"A0739AC6-76A0-DA11-8F1C-0014220EE585");

        private static OrganizationServiceProxy GetCrmServiceProxy(Uri uri, ClientCredentials clientCredentials)
        {
            OrganizationServiceProxy proxy = new OrganizationServiceProxy(uri, null, clientCredentials, null);
            if (proxy == null)
            {
                throw new Exception("Error: Crm connection service proxy is null. Check service Url and logon credentials");
            }
            return proxy;
        }

        public static EntityCollection GetContacts(Uri uri, ClientCredentials clientCredentials, List<string> columns)
        {
            ColumnSet columnSet = new ColumnSet(columns.ToArray());

            QueryExpression query = new QueryExpression();
            query.EntityName = "contact";
            query.ColumnSet = columnSet;
            query.Criteria = new FilterExpression(LogicalOperator.And);

            FilterExpression filter = new FilterExpression(LogicalOperator.And);
            filter.Conditions.Add(new ConditionExpression("contactid", ConditionOperator.NotNull));

            //FilterExpression filter1 = new FilterExpression(LogicalOperator.And);
            //filter1.Conditions.Add(new ConditionExpression("statuscode", ConditionOperator.NotEqual, 2));

            FilterExpression filter2 = new FilterExpression(LogicalOperator.And);
            filter2.Conditions.Add(new ConditionExpression("emailaddress1", ConditionOperator.NotNull));

            query.Criteria.Filters.AddRange(new FilterExpression[] {
                filter,
                //filter1,
                filter2
            });

            return RetrieveMultiple(query, uri, clientCredentials);
        }        

        public static EntityCollection GetAccounts(Uri uri, ClientCredentials clientCredentials)
        {
            string[] columns = {
                "accountid",
                "accountcategorycode",
                "name",
                "nn_primarytradecodeno",
                "new_partnertype",
                "new_erpsystem",
                "new_country",
                "address1_city",
                "address1_postalcode",
                "new_primarylineofbusiness",
                "accountnumber"
            };
            ColumnSet columnSet = new ColumnSet(columns);

            QueryExpression query = new QueryExpression();
            query.EntityName = "account";
            query.ColumnSet = columnSet;
            query.Criteria = new FilterExpression(LogicalOperator.And);

            FilterExpression filter = new FilterExpression(LogicalOperator.And);
            filter.Conditions.Add(new ConditionExpression("accountid", ConditionOperator.NotNull));

            LinkEntity le = new LinkEntity();
            le.LinkFromEntityName = "account";
            le.LinkFromAttributeName = "accountid";
            le.LinkToEntityName = "contact";
            le.LinkToAttributeName = "accountid";

            ConditionExpression conditionExpression = new ConditionExpression("statuscode", ConditionOperator.Equal, 1);

            le.LinkCriteria = new FilterExpression();
            le.LinkCriteria.FilterOperator = LogicalOperator.And;
            le.LinkCriteria.Conditions.Add(conditionExpression);

            query.LinkEntities.Add(le);
            query.Criteria.Filters.AddRange(new FilterExpression[] {
                filter
            });

            return RetrieveMultiple(query, uri, clientCredentials);
        }

        public static EntityCollection GetExistingActivities(Uri uri, ClientCredentials clientCredentials)
        {
            ConditionExpression expression1 = new ConditionExpression("new_dwkey", ConditionOperator.NotNull);

            FilterExpression filter = new FilterExpression(LogicalOperator.And);
            filter.Conditions.Add(expression1);

            QueryExpression query = new QueryExpression("new_activitiesfromemailmarketing");
            query.ColumnSet = new ColumnSet(new string[] { "activityid", "new_dwkey", "statecode", "subject", "description" });
            query.Criteria = filter;
            query.AddOrder("createdon", OrderType.Descending);

            EntityCollection entities = RetrieveMultiple(query, uri, clientCredentials);
            return entities;
        }

        public static Guid AddActivity(Guid? contactID, string subject, string description, string dwKey, Uri uri, ClientCredentials clientCredentials, Guid? ownerid)
        {
            Entity entity = new Entity("new_activitiesfromemailmarketing");
            if (contactID.HasValue)
            {
                AddOrUpdateAttribute(entity, "regardingobjectid", new EntityReference("contact", contactID.Value));
                AddOrUpdateAttribute(entity, "new_contact", new EntityReference("contact", contactID.Value));                
            }

            if (!ownerid.HasValue)
            {
                ownerid = PartnerSupportDynamicwebGuid;
            }
            AddOrUpdateAttribute(entity, "ownerid", new EntityReference("systemuser", ownerid.Value));

            AddOrUpdateAttribute(entity, "subject", subject);
            AddOrUpdateAttribute(entity, "description", description);
            AddOrUpdateAttribute(entity, "new_dwkey", dwKey);
            return Create(entity, uri, clientCredentials);
        }

        public static EntityCollection RetrieveMultiple(QueryExpression query, Uri uri, ClientCredentials clientCredentials)
        {
            EntityCollection result = new EntityCollection();
            using (var proxy = GetCrmServiceProxy(uri, clientCredentials))
            {
                query.PageInfo = new PagingInfo();
                query.PageInfo.Count = 5000;
                query.PageInfo.PageNumber = 1;
                query.PageInfo.PagingCookie = null;

                while (true)
                {
                    EntityCollection collection = proxy.RetrieveMultiple(query);

                    if (collection.Entities?.Count > 0)
                    {
                        result.Entities.AddRange(collection.Entities);
                    }
                    if (collection.MoreRecords)
                    {
                        query.PageInfo.PageNumber++;
                        query.PageInfo.PagingCookie = collection.PagingCookie;
                    }
                    else
                    {
                        break;
                    }
                }
            }
            return result;
        }

        public static Entity Retrieve(string entityName, Guid id, ColumnSet columnSet, Uri uri, ClientCredentials clientCredentials)
        {
            Entity result = null;
            using (var proxy = GetCrmServiceProxy(uri, clientCredentials))
            {
                try
                {
                    result = proxy.Retrieve(entityName, id, columnSet);
                }
                catch
                {
                    result = null;
                }
            }
            return result;
        }

        public static Guid Create(Entity entity, Uri uri, ClientCredentials clientCredentials)
        {
            Guid result = Guid.Empty;
            using (var proxy = GetCrmServiceProxy(uri, clientCredentials))
            {
                result = proxy.Create(entity);
            }
            return result;
        }

        public static void Update(Entity entity, Uri uri, ClientCredentials clientCredentials)
        {
            using (var proxy = GetCrmServiceProxy(uri, clientCredentials))
            {
                proxy.Update(entity);
            }
        }

        public static void AddOrUpdateAttribute(Entity entity, string name, object value)
        {
            if (entity != null && entity.Attributes != null)
            {
                if (entity.Contains(name))
                {
                    entity[name] = value;
                }
                else
                {
                    entity.Attributes.Add(new KeyValuePair<string, object>(name, value));
                }
            }
        }
    }
}
