﻿using System;
using System.Text;
using Dynamicweb.DataIntegration.ProviderHelpers;
using Microsoft.Xrm.Sdk;
using System.Collections.Generic;

namespace CrmIntegration.Helpers
{
    internal class EmailMarketingHelper
    {
        internal static string GetLeadDWKey(string id)
        {
            return string.Format("LeadID:{0}", id);
        }

        internal static string GetLeadActivitySubject(LeadInfo li)
        {
            string ret = string.Empty;
            if (li != null)
            {
                if (!string.IsNullOrEmpty(li.CompanyName))
                {
                    ret = string.Format("{0} : Dynamicweb Lead : {1}", li.LastVisitDate.ToString("dd-MM-yyyy hh:mm"), li.CompanyName);
                }
                else
                {
                    ret = string.Format("{0} : Dynamicweb Lead", li.LastVisitDate.ToString("dd-MM-yyyy hh:mm"));
                }
            }
            return ret;
        }

        internal static string GetLeadActivityBody(LeadInfo leadInfo)
        {
            StringBuilder sb = new StringBuilder();
            if (!string.IsNullOrEmpty(leadInfo.CountryCode) || !string.IsNullOrEmpty(leadInfo.CountryName))
            {
                sb.Append("Country ");
                if (!string.IsNullOrEmpty(leadInfo.CountryCode))
                    sb.AppendFormat("Code: {0} ", leadInfo.CountryCode);
                if (!string.IsNullOrEmpty(leadInfo.CountryName))
                    sb.AppendFormat("Name: {0}", leadInfo.CountryName);
                sb.Append(Environment.NewLine);
            }
            if (!string.IsNullOrEmpty(leadInfo.CompanyName))
            {
                sb.AppendFormat("Company: {0}{1}", leadInfo.CompanyName, Environment.NewLine);
            }
            if (leadInfo.PageInfos.Count > 0)
            {
                sb.AppendFormat("Leads:{0}", Environment.NewLine);
                foreach (LeadPageInfo lpi in leadInfo.PageInfos)
                {
                    sb.AppendFormat("{0}: {1}{2}", lpi.PageName, lpi.PageUrl, Environment.NewLine);
                }
            }
            return sb.ToString();
        }

        internal static string GetEmailDWKey(string emailid, string contactid)
        {
            return string.Format("EmailID:{0}.ContactID:{1}", emailid, contactid);
        }

        internal static string GetEmailActivitySubject(DateTime date)
        {
            return string.Format("{0} : Email Marketing", date.ToString("dd-MM-yyyy hh:mm"));
        }

        internal static string GetEmailActivityBody(EmailInfo emailInfo)
        {
            StringBuilder sb = new StringBuilder();
            if (!string.IsNullOrEmpty(emailInfo.EmailSubject))
            {
                sb.AppendFormat("Subject: {0}{1}", emailInfo.EmailSubject, Environment.NewLine);
            }
            if (emailInfo.ClickedLinks.Count > 0)
            {
                sb.AppendFormat("Clicked links:{0}", Environment.NewLine);
                foreach (string link in emailInfo.ClickedLinks)
                {
                    sb.AppendFormat("{0}{1}", link, Environment.NewLine);
                }
            }
            string result = sb.ToString();
            if (result.Length > 2000)
            {
                result = result.Substring(0, 2000);
            }

            return result;
        }

        internal static Dictionary<string, Entity> GetActivitiesInfo(EntityCollection entities)
        {            
            Dictionary<string, Entity> ret = new Dictionary<string, Entity>();            
            if (entities.Entities != null && entities.Entities.Count > 0)
            {
                foreach (Entity e in entities.Entities)
                {
                    if (e.Contains("new_dwkey") && e["new_dwkey"] != null && !string.IsNullOrEmpty(e["new_dwkey"].ToString()) && !ret.ContainsKey(e["new_dwkey"].ToString()))
                    {
                        ret.Add(e["new_dwkey"].ToString(), e);
                    }
                }
            }
            return ret;
        }

        internal static Tuple<Guid, int, string, string> GetActivityInfo(Entity entity)
        {
            Tuple<Guid, int, string, string> ret = null;
            if (entity != null)
            {
                ret = new Tuple<Guid, int, string, string>(
                        entity.Id,
                        entity.Contains("statecode") ? ((OptionSetValue)entity["statecode"]).Value : 0,
                        entity.Contains("subject") && entity["subject"] != null ? entity["subject"].ToString() : null,
                        entity.Contains("description") && entity["description"] != null ? entity["description"].ToString() : null);
            }
            return ret;
        }
    }
}