﻿using Microsoft.Xrm.Sdk;

namespace CrmIntegration
{
    public static class EntityExtensions
    {
        public static string GetAttributeValue(this Entity entity, string attributeName)
        {
            if (entity != null && entity.Contains(attributeName) && entity[attributeName] != null)
            {
                if (entity[attributeName] is EntityReference)
                {
                    EntityReference reference = entity[attributeName] as EntityReference;
                    if (reference != null)
                    {
                        return reference.Id.ToString();
                    }
                    else
                    {
                        return entity.FormattedValues.Contains(attributeName) ? entity.FormattedValues[attributeName] : null;
                    }
                }
                else if(entity[attributeName] is OptionSetValue)
                {
                    return entity.FormattedValues.Contains(attributeName) ? entity.FormattedValues[attributeName] : null;
                }
                else if (entity[attributeName] is bool)
                {
                    return entity[attributeName].ToString();
                }
                else if (entity[attributeName] is System.DateTime)
                {
                    return ((System.DateTime)entity[attributeName]).ToString("yyyy-MM-dd HH:mm:ss");
                }
                else
                {
                    return entity.FormattedValues.Contains(attributeName) ? entity.FormattedValues[attributeName] : entity[attributeName].ToString();                    
                }
            }
            else
            {
                return null;
            }
        }

        public static string GetReferenceAttributeName(this Entity entity, string attributeName)
        {
            if (entity != null && entity.Contains(attributeName) && entity[attributeName] != null)
            {
                if (entity[attributeName] is EntityReference)
                {
                    EntityReference reference = entity[attributeName] as EntityReference;
                    if (reference != null)
                    {
                        return reference.Name.ToString();
                    }
                    else
                    {
                        return null;
                    }
                }
                else
                {
                    return entity[attributeName].ToString();
                }
            }
            else
            {
                return null;
            }
        }
    }
}
