﻿using CrmIntegration.NotificationSubscribers;
using Dynamicweb.Security.UserManagement;
using Microsoft.Xrm.Sdk;
using System;
using System.Xml;

namespace CrmIntegration.Helpers
{
    public class Helper
    {
        /// <summary>
        /// Checks if CRM contact already exists with the userName
        /// CrmIntegration.Helpers.Helper.IsUserNameAlreadyTaken(string userName)
        /// </summary>
        /// <param name="userName">User UserName</param>
        /// <returns>True if contact exists, otherwise false</returns>
        public static bool IsUserNameAlreadyTaken(string userName)
        {
            if (!string.IsNullOrEmpty(userName))
            {
                BaseNotificationSubscriber subscriber = new BaseNotificationSubscriber();
                CrmAddIn addIn = subscriber.GetFirstFoundCrmAddIn();
                if (addIn != null)
                {
                    var contact = subscriber.GetCrmContactByUserName(userName, addIn);
                    return contact != null;
                }
            }
            return false;
        }

        /// <summary>
        /// Gets CRM contact by email
        /// CrmIntegration.Helpers.Helper.GetContactByEmail(string email)
        /// </summary>
        /// <param name="email">email</param>
        /// <returns>CRM Contact entity object if found, otherwise null</returns>
        public static Entity GetContactByEmail(string email)
        {
            BaseNotificationSubscriber subscriber = new BaseNotificationSubscriber();
            CrmAddIn addIn = subscriber.GetFirstFoundCrmAddIn();
            if (addIn != null)
            {
                return subscriber.GetCrmContactByEmail(email, addIn);
            }
            return null;
        }
        //private static void AddCrmBooleanField(XmlDocument xDoc, XmlElement item, CrmBoolean crmField, string userField)
        //{
        //    if (crmField == null || crmField.IsNull)
        //    {
        //        item.AppendChild(GetIsNullColumnTag(xDoc, userField));
        //    }
        //    else
        //    {
        //        item.AppendChild(GetColumnTag(xDoc, userField, crmField.Value.ToString().ToLower()));
        //    }
        //}

        //private static void AddCrmDateTimeField(XmlDocument xDoc, XmlElement item, CrmDateTime crmField, string userField, bool setNeverExpire = true)
        //{
        //    if (crmField == null || crmField.IsNull)
        //    {
        //        if (setNeverExpire)
        //        {
        //            item.AppendChild(GetColumnTag(xDoc, userField, "2999-12-31 00:00:00"));
        //        }
        //        else
        //        {
        //            item.AppendChild(GetColumnTag(xDoc, userField, DateTime.Parse(crmField.Value).ToString("yyyy-MM-dd HH:mm:ss")));
        //        }
        //    }
        //    else
        //    {
        //        item.AppendChild(GetColumnTag(xDoc, userField, DateTime.Parse(crmField.Value).ToString("yyyy-MM-dd HH:mm:ss")));
        //    }
        //}

        public static XmlElement GetColumnTag(XmlDocument xDoc, string name, string value)
        {
            XmlElement column = xDoc.CreateElement("column");
            column.SetAttribute("columnName", name);
            if (value != null)
            {
                column.AppendChild(xDoc.CreateCDataSection(value));
            }
            else
            {
                column.SetAttribute("isNull", "true");
            }
            return column;
        }

        public static XmlElement GetIsNullColumnTag(XmlDocument xDoc, string name)
        {
            XmlElement column = xDoc.CreateElement("column");
            column.SetAttribute("columnName", name);
            column.SetAttribute("isNull", "true");
            return column;
        }
        public static bool? ParseBool(string text)
        {
            if (!string.IsNullOrEmpty(text))
            {
                if (string.Equals(text, "Aktiv", StringComparison.OrdinalIgnoreCase) ||
                    string.Equals(text, "Active", StringComparison.OrdinalIgnoreCase) ||
                    string.Equals(text, "true", StringComparison.OrdinalIgnoreCase))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            return false;
        }

        public static bool UpdateField(Entity contact, string attribute, string value)
        {
            if (string.IsNullOrEmpty(value) && !contact.Contains(attribute))
            {
                return false;
            }
            if (!contact.Contains(attribute) || !string.Equals(contact[attribute].ToString(), value, StringComparison.OrdinalIgnoreCase))
            {
                CrmConnector.AddOrUpdateAttribute(contact, attribute, value);
                return true;
            }
            return false;
        }

        public static bool UpdateField(Entity contact, string attribute, DateTime? value)
        {
            if (!value.HasValue)
            {
                if (contact.Contains(attribute) && contact[attribute] != null)
                {
                    CrmConnector.AddOrUpdateAttribute(contact, attribute, null);
                    return true;
                }
                return false;
            }
            else
            {
                if (!contact.Contains(attribute) ||
                (contact[attribute] is DateTime && ((DateTime)contact[attribute] != value.Value) && (value != DateTime.MinValue && contact[attribute] != null)))
                {
                    CrmConnector.AddOrUpdateAttribute(contact, attribute, value.Value);
                    return true;
                }
                return false;
            }
        }
    }
}
