﻿using CrmIntegration.Helpers;
using Dynamicweb.Security.UserManagement.Common.CustomFields;
using Microsoft.Xrm.Sdk;
using System.Collections;
using System.Collections.Generic;
using System.Xml;

namespace CrmIntegration
{
    public class XmlFormatter
    {
        /// <summary>
        /// Returns users and groups in xml format for import to Dynamicweb
        /// </summary>
        /// <param name="users"></param>
        /// <param name="groups"></param>
        /// <returns></returns>
        public static XmlDocument GetImportXml(EntityCollection users, string importGroupID, List<string> columns, Hashtable accounts)
        {
            XmlDocument xDoc = new XmlDocument();
            XmlDeclaration xDec = xDoc.CreateXmlDeclaration("1.0", "UTF-8", "yes");
            xDoc.AppendChild(xDec);

            XmlElement xRoot = xDoc.CreateElement("tables");
            
            //xRoot.AppendChild(AddCustomFieldsNode(xDoc));

            XmlElement usersNode = xDoc.CreateElement("table");
            usersNode.SetAttribute("tableName", "AccessUser");

            if (users.Entities != null && users.Entities.Count > 0)
            {
                foreach (Entity user in users.Entities)
                {
                    XmlElement item = xDoc.CreateElement("item");
                    item.SetAttribute("table", "AccessUser");

                    foreach (string attribute in columns)
                    {
                        switch (attribute)
                        {
                            case "donotbulkemail":
                                item.AppendChild(Helper.GetColumnTag(xDoc, "AccessUserNewsletterAllowed",
                                    string.IsNullOrEmpty(user.GetAttributeValue("donotbulkemail")) ? "false" : (!bool.Parse(user.GetAttributeValue("donotbulkemail"))).ToString().ToLower()));
                                item.AppendChild(Helper.GetColumnTag(xDoc, attribute, user.GetAttributeValue(attribute)));
                                break;
                            case "ownerid":
                                item.AppendChild(!string.IsNullOrEmpty(user.GetReferenceAttributeName("ownerid")) ? Helper.GetColumnTag(xDoc, "AccessUser_Owner", user.GetReferenceAttributeName("ownerid")) : Helper.GetIsNullColumnTag(xDoc, "AccessUser_Owner"));
                                item.AppendChild(Helper.GetColumnTag(xDoc, attribute, user.GetAttributeValue(attribute)));
                                break;
                            default:                                
                                item.AppendChild(Helper.GetColumnTag(xDoc, attribute, user.GetAttributeValue(attribute)));
                                break;
                        }
                    }
                    item.AppendChild(Helper.GetColumnTag(xDoc, "AccessUserGroups", importGroupID));
                    string fullName = user.GetAttributeValue("fullname");
                    if (string.IsNullOrEmpty(fullName))
                    {
                        fullName = GetUserFullName(user.GetAttributeValue("firstname"), user.GetAttributeValue("lastname"));
                    }
                    item.AppendChild(Helper.GetColumnTag(xDoc, "AccessUserName", fullName));

                    bool? stateCode = Helper.ParseBool(user.GetAttributeValue("statecode"));
                    item.AppendChild(Helper.GetColumnTag(xDoc, "AccessUserActive", stateCode.HasValue ? stateCode.ToString() : null));                    

                    FillAccountFields(user, xDoc, item, accounts);
                    usersNode.AppendChild(item);
                }
            }

            xRoot.AppendChild(usersNode);
            xDoc.AppendChild(xRoot);
            return xDoc;
        }

        private static void FillAccountFields(Entity user, XmlDocument xDoc, XmlElement item, Hashtable accounts)
        {
            AccountInfo accountInfo = null;
            string accountID = user.GetAttributeValue("parentcustomerid");
            if (!string.IsNullOrEmpty(accountID) && accounts.ContainsKey(accountID))
            {
                accountInfo = (AccountInfo)accounts[accountID];
            }

            // Account fields
            item.AppendChild((accountInfo != null && !string.IsNullOrEmpty(accountInfo.Name)) ?
                Helper.GetColumnTag(xDoc, "AccessUserCompany", accountInfo.Name) :
                Helper.GetIsNullColumnTag(xDoc, "AccessUserCompany"));

            item.AppendChild((accountInfo != null && !string.IsNullOrEmpty(accountInfo.Nacecode)) ?
                Helper.GetColumnTag(xDoc, "AccessUser_NaceCode", accountInfo.Nacecode) :
                Helper.GetIsNullColumnTag(xDoc, "AccessUser_NaceCode"));

            item.AppendChild((accountInfo != null && !string.IsNullOrEmpty(accountInfo.Zip)) ?
                Helper.GetColumnTag(xDoc, "AccessUserZip", accountInfo.Zip) :
                Helper.GetIsNullColumnTag(xDoc, "AccessUserZip"));

            item.AppendChild((accountInfo != null && !string.IsNullOrEmpty(accountInfo.City)) ?
                Helper.GetColumnTag(xDoc, "AccessUserCity", accountInfo.City) :
                Helper.GetIsNullColumnTag(xDoc, "AccessUserCity"));

            item.AppendChild((accountInfo != null && !string.IsNullOrEmpty(accountInfo.Country)) ?
                Helper.GetColumnTag(xDoc, "AccessUserCountry", accountInfo.Country) :
                Helper.GetIsNullColumnTag(xDoc, "AccessUserCountry"));

            item.AppendChild((accountInfo != null && !string.IsNullOrEmpty(accountInfo.Category)) ?
                Helper.GetColumnTag(xDoc, "AccessUser_ParentCategory", accountInfo.Category) :
                Helper.GetIsNullColumnTag(xDoc, "AccessUser_ParentCategory"));

            item.AppendChild((accountInfo != null && !string.IsNullOrEmpty(accountInfo.PartnerType)) ?
                Helper.GetColumnTag(xDoc, "AccessUser_ParentPartnerType", accountInfo.PartnerType) :
                Helper.GetIsNullColumnTag(xDoc, "AccessUser_ParentPartnerType"));

            item.AppendChild((accountInfo != null && !string.IsNullOrEmpty(accountInfo.ErpSystem)) ?
                Helper.GetColumnTag(xDoc, "AccessUser_ErpSystem", accountInfo.ErpSystem) :
                Helper.GetIsNullColumnTag(xDoc, "AccessUser_ErpSystem"));

            item.AppendChild((accountInfo != null && !string.IsNullOrEmpty(accountInfo.PrimaryLineOfBusiness)) ?
                Helper.GetColumnTag(xDoc, "AccessUser_PrimaryLineOfBusiness", accountInfo.PrimaryLineOfBusiness) :
                Helper.GetIsNullColumnTag(xDoc, "AccessUser_PrimaryLineOfBusiness"));

            item.AppendChild((accountInfo != null && !string.IsNullOrEmpty(accountInfo.Number)) ?
                Helper.GetColumnTag(xDoc, "AccessUser_PartnerAccountNumber", accountInfo.Number) :
                Helper.GetIsNullColumnTag(xDoc, "AccessUser_PartnerAccountNumber"));
        }

        private static XmlElement AddCustomFieldsNode(XmlDocument xDoc)
        {
            //XML for User Custom Fileds creation
            XmlElement configNode = xDoc.CreateElement("config");
            XmlElement cfNode = xDoc.CreateElement("userCustomField");

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Owner");
            cfNode.SetAttribute("systemName", "AccessUser_Owner");
            cfNode.SetAttribute("type", ((int)Types.Text).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Content and Marketing Implementation");
            cfNode.SetAttribute("systemName", "AccessUser_C1Certified");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Content and Marketing Implementation Expire");
            cfNode.SetAttribute("systemName", "AccessUser_C1CertifiedExpire");
            cfNode.SetAttribute("type", ((int)Types.Date).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Ecommerce and Marketing Implementation");
            cfNode.SetAttribute("systemName", "AccessUser_C2Certified");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Ecommerce and Marketing Implementation Expire");
            cfNode.SetAttribute("systemName", "AccessUser_C2CertifiedExpire");
            cfNode.SetAttribute("type", ((int)Types.Date).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Content, Marketing and Ecommerce Developer");
            cfNode.SetAttribute("systemName", "AccessUser_C3Certified");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Content, Marketing and Ecommerce Developer Expire");
            cfNode.SetAttribute("systemName", "AccessUser_C3CertifiedExpire");
            cfNode.SetAttribute("type", ((int)Types.Date).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Implementing integrations");
            cfNode.SetAttribute("systemName", "AccessUser_C4Certified");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Implementing integrations expire");
            cfNode.SetAttribute("systemName", "AccessUser_C4CertifiedExpire");
            cfNode.SetAttribute("type", ((int)Types.Date).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Certified Dynamicweb Professional");
            cfNode.SetAttribute("systemName", "AccessUser_C0Certified");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Certified Dynamicweb Professional Expire");
            cfNode.SetAttribute("systemName", "AccessUser_C0CertifiedExpire");
            cfNode.SetAttribute("type", ((int)Types.Date).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Email address valid");
            cfNode.SetAttribute("systemName", "AccessUser_EmailAddressValid");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "PAM newsletter");
            cfNode.SetAttribute("systemName", "AccessUser_PamNewsletter");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "VIP contact mail");
            cfNode.SetAttribute("systemName", "AccessUser_VIPContactMail");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "MVP");
            cfNode.SetAttribute("systemName", "AccessUser_MVP");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Dynamicweb employee");
            cfNode.SetAttribute("systemName", "AccessUser_DWEmployee");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "ERP System");
            cfNode.SetAttribute("systemName", "AccessUser_ErpSystem");
            cfNode.SetAttribute("type", ((int)Types.Text).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Nace Code");
            cfNode.SetAttribute("systemName", "AccessUser_NaceCode");
            cfNode.SetAttribute("type", ((int)Types.Text).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Parent Category");
            cfNode.SetAttribute("systemName", "AccessUser_ParentCategory");
            cfNode.SetAttribute("type", ((int)Types.Text).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Parent Partner Type");
            cfNode.SetAttribute("systemName", "AccessUser_ParentPartnerType");
            cfNode.SetAttribute("type", ((int)Types.Text).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Primary Line of business");
            cfNode.SetAttribute("systemName", "AccessUser_PrimaryLineOfBusiness");
            cfNode.SetAttribute("type", ((int)Types.Text).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "Partner Account Number");
            cfNode.SetAttribute("systemName", "AccessUser_PartnerAccountNumber");
            cfNode.SetAttribute("type", ((int)Types.Text).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "AccessUser dwsubjectdownload");
            cfNode.SetAttribute("systemName", "AccessUser_dwsubjectdownload");
            cfNode.SetAttribute("type", ((int)Types.Text).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "AccessUser downloadtimestamp");
            cfNode.SetAttribute("systemName", "AccessUser_downloadtimestamp");
            cfNode.SetAttribute("type", ((int)Types.Date).ToString());
            configNode.AppendChild(cfNode);

            cfNode = xDoc.CreateElement("userCustomField");
            cfNode.SetAttribute("name", "AccessUser unsubscribe newsletter");
            cfNode.SetAttribute("systemName", "AccessUser_unsubscribenewsletter");
            cfNode.SetAttribute("type", ((int)Types.Boolean).ToString());
            configNode.AppendChild(cfNode);

            return configNode;
        }

        internal static string GetUserFullName(string firstName, string lastName)
        {
            if (!string.IsNullOrEmpty(firstName) && !string.IsNullOrEmpty(lastName))
            {
                return string.Format("{0} {1}", firstName, lastName);
            }
            else
            {
                return string.IsNullOrEmpty(firstName) ? lastName : firstName;
            }
        }
    }
}
