﻿using Dynamicweb.Core;
using Dynamicweb.Extensibility.AddIns;
using Dynamicweb.Extensibility.Notifications;
using Dynamicweb.Scheduling;
using Dynamicweb.Security.UserManagement;
using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using System;
using System.Linq;

namespace CrmIntegration.NotificationSubscribers
{
    public class BaseNotificationSubscriber : NotificationSubscriber
    {
        private string[] UserColumns = new string[] { "contactid", "donotbulkemail",
                                "fullname", "firstname", "lastname", "jobtitle", "emailaddress1", "new_username", "new_password",
                            "new_gdprdwnewsletterconsent", "new_gdprdwnewsletterconsenttimestamp", "new_gdprconsentacquiredby", "new_gdprnewsletterchangedby",
                            "new_dwsubjectdownload", "new_downloadtimestamp", "new_unsubscribenewsletter"};
        protected internal CrmAddIn GetFirstFoundCrmAddIn()
        {
            TaskCollection tasks = new TaskCollection();
            foreach (Dynamicweb.Scheduling.Task task in tasks)
            {
                if (task != null && task.AddInSettings != null && !string.IsNullOrEmpty(task.Assembly) && string.Compare(task.Assembly, typeof(CrmAddIn).FullName, true) == 0)
                {
                    CrmAddIn crmAddIn = (CrmAddIn)AddInManager.CreateAddInInstance(typeof(CrmAddIn));
                    if (crmAddIn != null)
                    {
                        crmAddIn.LoadParametersFromXml(task.AddInSettings);
                        if (crmAddIn.ClientCredentials != null && !string.IsNullOrEmpty(crmAddIn.WebServiceUrl))
                        {
                            return crmAddIn;
                        }
                    }
                }
            }
            return null;
        }

        protected Entity GetCrmContactById(Guid id, CrmAddIn addIn)
        {
            return CrmConnector.Retrieve("contact", id, new ColumnSet(UserColumns), new Uri(addIn.WebServiceUrl), addIn.ClientCredentials);
        }

        protected internal Entity GetCrmContactByEmail(string email, CrmAddIn addIn)
        {
            ColumnSet columnSet = new ColumnSet(UserColumns);

            QueryExpression query = new QueryExpression();
            query.EntityName = "contact";
            query.ColumnSet = columnSet;
            query.Criteria = new FilterExpression(LogicalOperator.And);

            FilterExpression filter = new FilterExpression(LogicalOperator.And);
            filter.Conditions.Add(new ConditionExpression("contactid", ConditionOperator.NotNull));

            FilterExpression filter2 = new FilterExpression(LogicalOperator.And);
            filter2.Conditions.Add(new ConditionExpression("emailaddress1", ConditionOperator.Equal, email));

            query.Criteria.Filters.AddRange(new FilterExpression[] {
                filter,
                filter2
            });

            var results = CrmConnector.RetrieveMultiple(query, new Uri(addIn.WebServiceUrl), addIn.ClientCredentials);
            return results?.Entities?.FirstOrDefault();
        }

        protected internal Entity GetCrmContactByUserName(string userName, CrmAddIn addIn)
        {
                ColumnSet columnSet = new ColumnSet(UserColumns);

                QueryExpression query = new QueryExpression();
                query.EntityName = "contact";
                query.ColumnSet = columnSet;
                query.Criteria = new FilterExpression(LogicalOperator.And);

                FilterExpression filter = new FilterExpression(LogicalOperator.And);
                filter.Conditions.Add(new ConditionExpression("contactid", ConditionOperator.NotNull));

                FilterExpression filter2 = new FilterExpression(LogicalOperator.And);
                filter2.Conditions.Add(new ConditionExpression("new_username", ConditionOperator.Equal, userName));

                query.Criteria.Filters.AddRange(new FilterExpression[] {
                filter,
                filter2
            });

                var results = CrmConnector.RetrieveMultiple(query, new Uri(addIn.WebServiceUrl), addIn.ClientCredentials);
                return results?.Entities?.FirstOrDefault();
        }

        protected bool UpdateGdprdwnewsletterconsent(bool gdprNewsletterConsent, Entity contact)
        {
            bool updated = false;            
            if (!contact.Contains("new_gdprdwnewsletterconsent") || (bool)contact["new_gdprdwnewsletterconsent"] != gdprNewsletterConsent)
            {
                updated = true;
                CrmConnector.AddOrUpdateAttribute(contact, "new_gdprdwnewsletterconsent", gdprNewsletterConsent);
                int websiteOptionId = 100000005; //Website option value from the CRM dropdown                
                CrmConnector.AddOrUpdateAttribute(contact, "new_gdprconsentacquiredby", new OptionSetValue(websiteOptionId));
                Guid administratorGuid = new Guid("c5504909-5524-e611-80e3-5065f38aea71");
                CrmConnector.AddOrUpdateAttribute(contact, "new_gdprnewsletterchangedby", new EntityReference("contact", administratorGuid));
                CrmConnector.AddOrUpdateAttribute(contact, "new_gdprdwnewsletterconsenttimestamp", DateTime.Now);
            }
            return updated;
        }

        protected object GetCustomFiledValue(User user, string customFieldName)
        {
            object result = null;
            if (user.CustomFieldValues != null)
            {
                var customFieldValue = user.CustomFieldValues.FirstOrDefault(cfv => cfv.CustomField != null && string.Equals(cfv.CustomField.SystemName, customFieldName, StringComparison.OrdinalIgnoreCase));
                result = customFieldValue?.Value;
            }
            return result;
        }
    }
}
