﻿using System;
using Dynamicweb.Extensibility.Notifications;
using Microsoft.Xrm.Sdk;
using Dynamicweb.Logging;
using Dynamicweb.Core;
using Dynamicweb.Security.UserManagement;
using CrmIntegration.Helpers;

namespace CrmIntegration.NotificationSubscribers
{
    public class ConsentNotificationSubscriber : BaseNotificationSubscriber
    {
        public void OnNotify(Dynamicweb.DataProcessing.Notifications.ConsentNotificationArgs args)
        {
            if (args != null)
            {                
                if (args != null && args.Consent != null)
                {
                    ILogger logger = LogManager.Current.GetLogger("Users");
                    CrmAddIn addIn = GetFirstFoundCrmAddIn();

                    if (addIn != null)
                    {                        
                        if (string.Equals(args.Consent.ActivityId, addIn.EmailAllowedConsentActivity, StringComparison.OrdinalIgnoreCase))
                        {
                            if (args.Consent.SubjectType == "User" || args.Consent.SubjectType == "Email")
                            {
                                User user = null;
                                string email = null;
                                if (args.Consent.SubjectType == "User")
                                {
                                    int userId = Converter.ToInt32(args.Consent.SubjectId);
                                    user = User.GetUserByID(userId);
                                    if (user != null)
                                    {
                                        email = user.Email;
                                    }
                                }
                                else if (args.Consent.SubjectType == "Email")
                                {
                                    email = args.Consent.SubjectId;
                                    if (!string.IsNullOrEmpty(email))
                                    {
                                        user = User.GetUserByEmailAddress(email);
                                    }
                                }

                                Entity contact = null;
                                if (user != null)
                                {
                                    if (!string.IsNullOrEmpty(user.ExternalID))
                                    {
                                        Guid id = Guid.Empty;
                                        if (Guid.TryParse(user.ExternalID, out id))
                                        {
                                            contact = GetCrmContactById(id, addIn);
                                        }
                                    }
                                }
                                if (contact == null && !string.IsNullOrEmpty(email))
                                {
                                    contact = GetCrmContactByEmail(email, addIn);
                                }
                                bool gdprdwnewsletterconsent = args.Consent.Status == Dynamicweb.DataProcessing.ConsentStatus.Given;
                                if (contact != null)
                                {
                                    bool update = UpdateGdprdwnewsletterconsent(gdprdwnewsletterconsent, contact);

                                    if (!string.IsNullOrEmpty(addIn.SubjectDownloadId) && !string.IsNullOrEmpty(Dynamicweb.Context.Current.Request[addIn.SubjectDownloadId]))
                                    {
                                        update = Helper.UpdateField(contact, "new_dwsubjectdownload", Dynamicweb.Context.Current.Request[addIn.SubjectDownloadId]) ? true : update;                                        
                                    }
                                    if (!string.IsNullOrEmpty(addIn.DownloadTimestampId) && !string.IsNullOrEmpty(Dynamicweb.Context.Current.Request[addIn.DownloadTimestampId]))
                                    {
                                        update = Helper.UpdateField(contact, "new_downloadtimestamp", Converter.ToDateTime(Dynamicweb.Context.Current.Request[addIn.DownloadTimestampId])) ? true : update;                                        
                                    }

                                    if (update)
                                    {
                                        CrmConnector.Update(contact, new Uri(addIn.WebServiceUrl), addIn.ClientCredentials);
                                        logger?.Log($"Crm contact with id='{contact.Id.ToString()}' was updated.");
                                    }
                                }
                                else
                                {
                                    if (!string.IsNullOrEmpty(email))
                                    {
                                        CreateNewCrmContact(user, email, gdprdwnewsletterconsent, addIn);                                        
                                    }
                                }
                            }
                        }
                    }
                    else
                    {
                        logger?.Error("CrmAddIn with not empty WebServiceUrl, UserName, Password not found in the Scheduled tasks.");
                    }
                }
            }
        }

        private void CreateNewCrmContact(User user, string email, bool gdprdwnewsletterconsent, CrmAddIn addIn)
        {
            //new user for CRM     
            Entity contact = new Entity("contact");
            CrmConnector.AddOrUpdateAttribute(contact, "emailaddress1", email);
            CrmConnector.AddOrUpdateAttribute(contact, "new_username", email);
            CrmConnector.AddOrUpdateAttribute(contact, "new_newuserfromconsentform", true);
            UpdateGdprdwnewsletterconsent(gdprdwnewsletterconsent, contact);

            if (user != null)
            {
                if (!string.IsNullOrEmpty(user.UserName))
                {
                    CrmConnector.AddOrUpdateAttribute(contact, "new_username", user.UserName);
                }
                CrmConnector.AddOrUpdateAttribute(contact, "firstname",
                    !string.IsNullOrEmpty(user.FirstName) ? user.FirstName : user.Name);
                CrmConnector.AddOrUpdateAttribute(contact, "lastname", user.LastName);
            }
            else
            {
                if (Dynamicweb.Context.Current != null && Dynamicweb.Context.Current.Request != null)
                {
                    string name = Dynamicweb.Context.Current.Request["Name"];
                    if (!string.IsNullOrEmpty(name))
                    {
                        CrmConnector.AddOrUpdateAttribute(contact, "new_username", name);
                        CrmConnector.AddOrUpdateAttribute(contact, "firstname", name);
                    }
                }
            }
            if(!string.IsNullOrEmpty(addIn.SubjectDownloadId) && !string.IsNullOrEmpty(Dynamicweb.Context.Current.Request[addIn.SubjectDownloadId]))
            {
                CrmConnector.AddOrUpdateAttribute(contact, "new_dwsubjectdownload", Dynamicweb.Context.Current.Request[addIn.SubjectDownloadId]);
            }
            if (!string.IsNullOrEmpty(addIn.DownloadTimestampId) && !string.IsNullOrEmpty(Dynamicweb.Context.Current.Request[addIn.DownloadTimestampId]))
            {
                var date = Converter.ToDateTime(Dynamicweb.Context.Current.Request[addIn.DownloadTimestampId]);
                if (date != DateTime.MinValue)
                {
                    CrmConnector.AddOrUpdateAttribute(contact, "new_downloadtimestamp", date);
                }
            }
            CrmConnector.Create(contact, new Uri(addIn.WebServiceUrl), addIn.ClientCredentials);
        }
    }
}
