﻿using Dynamicweb;
using Dynamicweb.Extensibility.Notifications;
using Dynamicweb.Forms;
using Dynamicweb.Security.UserManagement;
using Dynamicweb.SystemTools.Security;
using Microsoft.Xrm.Sdk;
using System;
using System.Collections.Generic;
using System.Linq;

namespace CrmIntegration.NotificationSubscribers
{
    [Subscribe(Notifications.Frontend.OnBeforeContent)]
    public class FormsForEditorsOnBeforeContent : BaseNotificationSubscriber
    {
        public override void OnNotify(string notification, NotificationArgs args)
        {
            if (args != null)
            {
                Notifications.Frontend.OnBeforeContentArgs myArgs = (Notifications.Frontend.OnBeforeContentArgs)args;

                if (myArgs != null && myArgs.ContentModule != null)
                {
                    if (!string.IsNullOrEmpty(myArgs.ContentModule.RequestContext("cmd")))
                    {
                        if (myArgs.ContentModule.RequestContext("cmd").Equals("save", StringComparison.OrdinalIgnoreCase)
                            && Context.Current.Request.HttpMethod.Equals("post", StringComparison.OrdinalIgnoreCase))
                        {
                            CrmAddIn addIn = GetFirstFoundCrmAddIn();
                            if (addIn != null && !string.IsNullOrEmpty(addIn.RegisterNewUserForm))
                            {
                                FormAntispam.SpamCheck();
                                Form f = Form.GetFormById(Dynamicweb.Core.Converter.ToInt32(myArgs.ContentModule.Properties.get_Value("FormID"))); //This one must not be overridable

                                if (f != null && f.ID == Dynamicweb.Core.Converter.ToInt32(addIn.RegisterNewUserForm))
                                {
                                    if (myArgs.ContentModule.MaxSubmitsReached(f))
                                    {
                                        myArgs.StopExecution = true;
                                        myArgs.Output = myArgs.ContentModule.HandleMaxSubmitCountAction();
                                        return;
                                    }

                                    List<SubmitData> fieldValues = new List<SubmitData>();

                                    foreach (Field currentField in f.Fields)
                                    {
                                        SubmitData data = new SubmitData()
                                        {
                                            FieldID = currentField.ID,
                                            Fieldname = currentField.Name
                                        };

                                        if (currentField.Fieldtype == FieldType.File)
                                        {
                                            data.Value = myArgs.ContentModule.SaveFormFiles(currentField);
                                        }
                                        else
                                        {
                                            data.Value = myArgs.ContentModule.RequestContext(currentField.SystemName);
                                        }
                                        fieldValues.Add(data);
                                    }

                                    string result = ProcessForm(fieldValues, addIn);
                                    if (!string.IsNullOrEmpty(result))
                                    {
                                        myArgs.StopExecution = true;
                                        result = $"User is not created <br /> Error information:<br />{result}";
                                        myArgs.Output = result;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        private string ProcessForm(List<SubmitData> fieldValues, CrmAddIn addIn)
        {
            string error = null;
            var currentUser = Dynamicweb.Security.UserManagement.User.GetCurrentExtranetUser();
            if (currentUser != null)
            {
                Guid accountId;
                Guid.TryParse(currentUser.CustomerNumber, out accountId);
                if (accountId != Guid.Empty)
                {
                    //if (addIn != null)
                    //{
                    string pwd = GetFieldValue("Desiredpassword", fieldValues);
                    string confirmPwd = GetFieldValue("Confirmdesiredpassword", fieldValues);
                    if (!string.Equals(pwd, confirmPwd))
                    {
                        return "Desired password and Confirm desired password must match";
                    }
                    string email = GetFieldValue("Email", fieldValues);
                    if (!string.IsNullOrEmpty(email))
                    {
                        if (GetCrmContactByEmail(email, addIn) != null)
                        {
                            return $"User with '{email}' email already exists";
                        }
                        string userName = GetFieldValue("Desiredusername", fieldValues);
                        if (!string.IsNullOrEmpty(userName))
                        {
                            if (GetCrmContactByUserName(userName, addIn) != null)
                            {
                                return $"User with the user name {userName} already exists";
                            }
                            User user = new User();
                            user.Email = email;
                            user.UserName = userName;
                            if (!user.IsUserNameValid())
                            {
                                return $"User with the user name {userName} already exists in Dynamicweb";
                            }
                            Guid externaId;
                            error = CreateNewCrmContact(email, userName, accountId, fieldValues, addIn, out externaId);
                            if (externaId != Guid.Empty)
                            {
                                //Create new DW user
                                int group = Dynamicweb.Core.Converter.ToInt32(addIn.DestinationFolder.Replace("GRP_", ""));
                                if (group > 0)
                                {
                                    user.AddToGroup(group);
                                }
                                FillUser(user, fieldValues, accountId, externaId);
                                Context.Current.Items["FormSubmitUserCreation"] = true;
                                bool created = user.Save();
                                Context.Current.Items.Remove("FormSubmitUserCreation");
                                if (!created)
                                {
                                    error += $" Can not create the user in Dynamicweb";
                                }
                            }
                        }
                    }
                    else
                    {
                        error = "Email can not be empty";
                    }
                    //}
                    //else
                    //{
                    //    error = $"Create and configure CrmAddIn in Settings -> Integration -> Integration Framework Batch";
                    //}
                }
                else
                {
                    error = $"Parent User with Id:{currentUser.ID} and UserName: {currentUser.UserName} has invalid customer number: {currentUser.CustomerNumber}";
                }
            }
            else
            {
                error = "Parent user must be logged in before creating a new user";
            }
            return error;
        }

        private string GetFieldValue(string systemName, List<SubmitData> fieldValues)
        {
            var fieldValue = fieldValues.FirstOrDefault(fv => fv.Field != null && string.Equals(systemName, fv.Field.SystemName, StringComparison.OrdinalIgnoreCase));
            //if(fieldValue != null && fieldValue.Field != null && fieldValue.Field.Type == "Select" && !string.IsNullOrEmpty(fieldValue.Value))
            //{

            //}
            return fieldValue?.Value;
        }

        private string CreateNewCrmContact(string email, string userName, Guid parentAccountId, List<SubmitData> fieldValues, CrmAddIn addIn, out Guid id)
        {
            id = new Guid();
            //new user for CRM     
            Entity contact = new Entity("contact");
            CrmConnector.AddOrUpdateAttribute(contact, "emailaddress1", email);
            CrmConnector.AddOrUpdateAttribute(contact, "new_username", userName);

            CrmConnector.AddOrUpdateAttribute(contact, "firstname", GetFieldValue("Name", fieldValues));
            CrmConnector.AddOrUpdateAttribute(contact, "lastname", GetFieldValue("Surname", fieldValues));

            CrmConnector.AddOrUpdateAttribute(contact, "jobtitle", GetFieldValue("Title", fieldValues));
            CrmConnector.AddOrUpdateAttribute(contact, "new_password", GetFieldValue("Desiredpassword", fieldValues));
            CrmConnector.AddOrUpdateAttribute(contact, "donotbulkemail", !Dynamicweb.Core.Converter.ToBoolean(GetFieldValue("CheckhereifDynamicwebmaysendyouemails", fieldValues)));

            //Find customer
            CrmConnector.AddOrUpdateAttribute(contact, "parentcustomerid", new EntityReference("account", parentAccountId));

            try
            {
                id = CrmConnector.Create(contact, new Uri(addIn.WebServiceUrl), addIn.ClientCredentials);
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
            return null;
        }

        private void FillUser(User user, List<SubmitData> fieldValues, Guid accountId, Guid externaId)
        {
            string firstName = GetFieldValue("Name", fieldValues);
            string lastName = GetFieldValue("Surname", fieldValues);
            user.Name = XmlFormatter.GetUserFullName(firstName, lastName);
            user.FirstName = firstName;
            user.LastName = lastName;
            user.JobTitle = GetFieldValue("Title", fieldValues);
            user.Password = GetFieldValue("Desiredpassword", fieldValues);
            user.CustomerNumber = accountId.ToString();
            user.Company = GetFieldValue("Company", fieldValues);
            user.ExternalID = externaId.ToString();
            user.EmailAllowed = Dynamicweb.Core.Converter.ToBoolean(GetFieldValue("CheckhereifDynamicwebmaysendyouemails", fieldValues));
            user.Active = true;
        }
    }
}
