﻿using System;
using Dynamicweb.Extensibility.Notifications;
using Microsoft.Xrm.Sdk;
using Dynamicweb.Logging;
using Dynamicweb.Core;
using Dynamicweb.Security.SystemTools;
using Dynamicweb.Security.UserManagement;
using Dynamicweb.Environment;
using Dynamicweb.Data;
using CrmIntegration.Helpers;

namespace CrmIntegration.NotificationSubscribers
{
    [Subscribe(Dynamicweb.Security.UserManagement.Notifications.Notifications.UserSaved)]
    public class UserSaved : BaseNotificationSubscriber
    {
        public override void OnNotify(string notification, NotificationArgs args)
        {
            if (args != null && !Converter.ToBoolean(Dynamicweb.Context.Current.Items["FormSubmitUserCreation"]))
            {
                Dynamicweb.Security.UserManagement.Notifications.UserNotificationArgs myArgs = (Dynamicweb.Security.UserManagement.Notifications.UserNotificationArgs)args;

                Guid id = Guid.Empty;
                if (myArgs != null && myArgs.Subject != null && !string.IsNullOrEmpty(myArgs.Subject.ExternalID)
                    && Guid.TryParse(myArgs.Subject.ExternalID, out id))
                {
                    ILogger logger = LogManager.Current.GetLogger("Users");
                    CrmAddIn addIn = GetFirstFoundCrmAddIn();

                    if (addIn != null)
                    {
                        Entity contact = GetCrmContactById(id, addIn);
                        if (contact != null)
                        {
                            bool update = false;
                            if (!contact.Contains("donotbulkemail") || (bool)contact["donotbulkemail"] != !myArgs.Subject.EmailAllowed)
                            {
                                update = true;
                                CrmConnector.AddOrUpdateAttribute(contact, "donotbulkemail", !myArgs.Subject.EmailAllowed);
                            }
                            update = Helper.UpdateField(contact, "fullname", myArgs.Subject.Name) ? true : update;
                            update = Helper.UpdateField(contact, "firstname", myArgs.Subject.FirstName) ? true : update;
                            update = Helper.UpdateField(contact, "lastname", myArgs.Subject.LastName) ? true : update;
                            update = Helper.UpdateField(contact, "jobtitle", myArgs.Subject.Title) ? true : update;
                            update = Helper.UpdateField(contact, "emailaddress1", myArgs.Subject.Email) ? true : update;
                            update = Helper.UpdateField(contact, "new_username", myArgs.Subject.UserName) ? true : update;
                            update = UpdatePasswordField(contact, myArgs.Subject) ? true : update;
                            update = UpdateGdprdwnewsletterconsent(Converter.ToBoolean(GetCustomFiledValue(myArgs.Subject, "AccessUser_GDPRNewsletterConsent")), contact) ? true : update; ;
                            update = Helper.UpdateField(contact, "new_dwsubjectdownload", Converter.ToString(GetCustomFiledValue(myArgs.Subject, "AccessUser_dwsubjectdownload"))) ? true : update; ;
                            object value = GetCustomFiledValue(myArgs.Subject, "AccessUser_downloadtimestamp");
                            update = Helper.UpdateField(contact, "new_downloadtimestamp", 
                                (value is DBNull || value == null) ? (DateTime?)null : Converter.ToDateTime(value)) ? true : update;
                            //send new_unsubscribenewsletter = true to CRM
                            if (!contact.Contains("new_unsubscribenewsletter") || !(bool)contact["new_unsubscribenewsletter"])
                            {
                                update = true;
                                CrmConnector.AddOrUpdateAttribute(contact, "new_unsubscribenewsletter", true);
                            }
                            if (update)
                            {
                                CrmConnector.Update(contact, new Uri(addIn.WebServiceUrl), addIn.ClientCredentials);
                                logger?.Log($"Crm contact with id='{id}' was updated. AccessUserId={myArgs.Subject.ID}.");
                            }
                        }
                        else
                        {
                            logger?.Log($"Crm contact with id='{id}' not found. AccessUserId={myArgs.Subject.ID}.");
                        }
                    }
                    else
                    {
                        logger?.Error("CrmAddIn with not empty WebServiceUrl, UserName, Password not found in the Scheduled tasks.");
                    }
                }
            }
        }        

        private string GetSubmittedUserPassword()
        {
            string password = null;
            if (ExecutingContext.IsBackEnd())
            {
                password = Dynamicweb.Context.Current.Request["Password"];                
            }
            else
            {
                password = Database.SqlEscapeInjection(Dynamicweb.Context.Current.Request["UserManagement_Form_NewPassword"], ExecutingContext.IsFrontEnd());
            }            
            return password;
        }

        private bool UpdatePasswordField(Entity contact, User user)
        {
            string password = null;            
            if (IsPasswordEncrypted(user.Password))
            {
                string submittedPassword = GetSubmittedUserPassword();
                if (!string.IsNullOrEmpty(submittedPassword) && EncryptPassword(submittedPassword) == user.Password)
                {
                    password = submittedPassword;
                }                
            }
            else
            {
                password = user.Password;
            }
            if (password != null)
            {
                string attribute = "new_password";
                if (string.IsNullOrEmpty(password) && !contact.Contains(attribute))
                {
                    return false;
                }
                if (!contact.Contains(attribute) || !string.Equals(contact[attribute].ToString(), password, StringComparison.OrdinalIgnoreCase))
                {
                    CrmConnector.AddOrUpdateAttribute(contact, attribute, password);
                    return true;
                }
            }
            return false;
        }        

        private string EncryptPassword(string password)
        {
            var hashAlgorithm = UserPasswordHashAlgorithm.MD5;
            Enum.TryParse(Dynamicweb.Configuration.SystemConfiguration.Instance.GetValue("/Globalsettings/Modules/Extranet/EncryptPasswordHash"), out hashAlgorithm);
            return Crypto.EncryptPassword(password, hashAlgorithm);
        }

        private bool IsPasswordEncrypted(string password)
        {
            return (password.Length == 32 || password.Length == 128);
        }                
    }
}
